//
//  HSVPolarisingFilter.swift
//  Polar
//
//  Created by Paul Wallace on 15/03/2016.
//  Copyright © 2016 Ynformatics. All rights reserved.
//

import Foundation
import UIKit



class HSVPolarisingFilter: CIFilter {
    private static var kernel: CIColorKernel?
    private static var context: CIContext?
    
    private var _inputImage0: CIImage?
    var inputImage0: CIImage? {
        get { return _inputImage0 }
        set { _inputImage0 = newValue }
    }
    private var _inputImage45: CIImage?
    var inputImage45: CIImage? {
        get { return _inputImage45 }
        set { _inputImage45 = newValue }
    }
    private var _inputImage90: CIImage?
    var inputImage90: CIImage? {
        get { return _inputImage90 }
        set { _inputImage90 = newValue }
    }
    
    override init() {
        super.init()
      
        HSVPolarisingFilter.preload()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
    }
    
    override var outputImage : CIImage! {
        if  let inputImage0 = inputImage0,
            let inputImage45 = inputImage45,
            let inputImage90 = inputImage90,
            let kernel = HSVPolarisingFilter.kernel {
                return kernel.applyWithExtent(inputImage0.extent, roiCallback: { (_, _) -> CGRect in return inputImage0.extent  }, arguments: [inputImage0, inputImage45, inputImage90 ]) // to support iOS8
                // return kernel.applyWithExtent(inputImage.extent, arguments: [inputImage, fc.red, fc.green, fc.blue]) // iOS9 and newer
        }
        return nil
    }
    
    func outputUIImage() -> UIImage {
        let ciimage = self.outputImage
        
        return UIImage(CGImage: HSVPolarisingFilter.context!.createCGImage(ciimage, fromRect: ciimage.extent))
    }
    
    private class func createKernel() -> CIColorKernel {
        let kernelString = try! String(contentsOfFile: NSBundle.mainBundle().pathForResource("HSVPolarisingFilterKernel", ofType: "cikernel")!, encoding: NSUTF8StringEncoding)
        
        return CIColorKernel(string: kernelString)!
    }
    
    class func preload() {
        // preloading kernel speeds up first execution of filter
        if kernel != nil {
            return
        }
        
       
        kernel = createKernel()
        
        context = CIContext(options: [kCIContextWorkingColorSpace: NSNull()])
    }
}