
import UIKit
import AVFoundation
import CoreMedia


class ViewController: UIViewController, AVCaptureVideoDataOutputSampleBufferDelegate
{
    @IBOutlet weak var filterChooser: UISegmentedControl!
    @IBOutlet weak var imageView: UIImageView!
    
    @IBAction func click(sender: UIButton) {
        UIImageWriteToSavedPhotosAlbum(self.imageView.image!, nil, nil, nil)
    }
    
    @IBAction func shutterDelay(sender: UISlider) {
        shutterDelayValue = sender.value
    }
    
    @IBAction func Skipper(sender: UIStepper) {
        skip = Int(sender.value)
        frame = 0
    }
    
    @IBAction func calibrate(sender: UIButton)
    {
        let device = AVCaptureDevice.defaultDeviceWithMediaType(AVMediaTypeVideo)
        if(device.hasTorch)
        {
            do
            {
                try device.lockForConfiguration()
                if(device.torchMode == AVCaptureTorchMode.On)
                {
                    device.torchMode = AVCaptureTorchMode.Off
                }
                else
                {
                    try device.setTorchModeOnWithLevel(0.0001)
                    NSThread.sleepForTimeInterval(2)
                    device.torchMode = AVCaptureTorchMode.Off
                }
                device.unlockForConfiguration()
            }
            catch
            {
                print( error)
            }
            
        }
    }
    
    
    let delayControl = UISlider()
    var shutterDelayValue : Float = 0
   
    var frame :Int = 0
    var skip :Int = 0
    var framesToSkip : Int = 0;
    
    var cameraImage0:CIImage!
    var cameraImage1:CIImage!
    var cameraImage2:CIImage!
    
    var previousTime: NSTimeInterval = 0
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        imageView.contentMode = UIViewContentMode.ScaleAspectFit
        
        let captureSession = AVCaptureSession()
        captureSession.sessionPreset = AVCaptureSessionPresetPhoto
        
        let backCamera = AVCaptureDevice.defaultDeviceWithMediaType(AVMediaTypeVideo)
        
        do
        {
            let input = try AVCaptureDeviceInput(device: backCamera)
            
            captureSession.addInput(input)
        }
        catch
        {
            print("can't access camera")
            return
        }
        
        RGBPolarisingFilter.preload()
        HSVPolarisingFilter.preload()
        
        // although we don't use this, it's required to get captureOutput invoked
        let previewLayer = AVCaptureVideoPreviewLayer(session: captureSession)
        view.layer.addSublayer(previewLayer)
        
        let videoOutput = AVCaptureVideoDataOutput()
        
        videoOutput.setSampleBufferDelegate(self, queue: dispatch_queue_create("sample buffer delegate", DISPATCH_QUEUE_SERIAL))
        if captureSession.canAddOutput(videoOutput)
        {
            captureSession.addOutput(videoOutput)
        }
        
        UIApplication.sharedApplication().idleTimerDisabled = true
        
        captureSession.startRunning()
    }
    
    override func viewDidDisappear(animated: Bool) {
        UIApplication.sharedApplication().idleTimerDisabled = false
    }
    
    func captureOutput(captureOutput: AVCaptureOutput!, didOutputSampleBuffer sampleBuffer: CMSampleBuffer!, fromConnection connection: AVCaptureConnection!)
    {
        
        let pixelBuffer = CMSampleBufferGetImageBuffer(sampleBuffer)
        
        if( framesToSkip > 0)
        {
            framesToSkip -= 1
            
            return
        }
        
        if (frame == 0)
        {
            let currentTime = NSDate.timeIntervalSinceReferenceDate()
            
            if(currentTime - previousTime < 0.180)
            {
                skip += 1
            }
            
            previousTime = currentTime
            
            cameraImage0 = CIImage(CVPixelBuffer: pixelBuffer!)
            frame = 1
            
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, Int64(Double(shutterDelayValue) * Double(NSEC_PER_MSEC))),
                           dispatch_get_main_queue())
            {
                self.flash()
            }
            
            framesToSkip = skip
            
            return
        }
        
        if (frame == 1)
        {
            cameraImage1 = CIImage(CVPixelBuffer: pixelBuffer!)
            frame = 2
            framesToSkip = skip
            
            return
        }
        
        if (frame == 2)
        {
            cameraImage2 = CIImage(CVPixelBuffer: pixelBuffer!)
            frame = 0
            framesToSkip = skip
            
            if(filterChooser.selectedSegmentIndex == 0)
            {
                
                let pf =  RGBPolarisingFilter( )
                
                pf.inputImage0 = cameraImage0
                pf.inputImage45 = cameraImage1
                pf.inputImage90 = cameraImage2
                
                let filteredImage = pf.outputUIImage()
                dispatch_async(dispatch_get_main_queue())
                {
                    self.imageView.image = filteredImage
                }
            }
            else{
                
                let pf =  HSVPolarisingFilter( )
                
                pf.inputImage0 = cameraImage0
                pf.inputImage45 = cameraImage1
                pf.inputImage90 = cameraImage2
                
                let filteredImage = pf.outputUIImage()
                dispatch_async(dispatch_get_main_queue())
                {
                    self.imageView.image = filteredImage
                }
            }
        
            return
        }
        
    }
    
    
    func flash()
    {
        let device = AVCaptureDevice.defaultDeviceWithMediaType(AVMediaTypeVideo)
        if(device.hasTorch)
        {
            do
            {
                try device.lockForConfiguration()
                if(device.torchMode == AVCaptureTorchMode.On)
                {
                    device.torchMode = AVCaptureTorchMode.Off
                }
                else
                {
                    try device.setTorchModeOnWithLevel(1)
                    //NSThread.sleepForTimeInterval(0.001)
                    device.torchMode = AVCaptureTorchMode.Off
                }
                device.unlockForConfiguration()
            }
            catch
            {
                print( error)
            }
            
        }
    }
}


